/**
 * ===================== SETTINGS ========================
 */
/**
 * Библиотеки CSS
 */
var libCSS = [
	// 'src/assets/fonts/stylesheet.css',
	// 'node_modules/bootstrap/dist/css/bootstrap.css'
];

/**
 * Версии поддерживаемых браузеров
 */
var platform = [ 'last 4 versions', 'ie >= 11', 'iOS >= 9' ];


/* ======================================================== */
/* global require */
var gulp = require( 'gulp' ); // Собственно gulp
var pump = require( 'pump' ); // отслеживание ошибок

var concat = require( 'gulp-concat' ); // объединение набора файлов в один
var postcss = require( 'gulp-postcss' ); // Пост-обработка CSS
var postcssPresetEnv = require( 'postcss-preset-env' ); // CSS Next
var cssvariables = require('postcss-css-variables');
var sourcemaps = require( 'gulp-sourcemaps' ); // карты исходного кода

var webpack = require( 'webpack-stream' );
var compiler = require( 'webpack' );

var imagemin = require( 'gulp-image' );  // оптимизация картинок
var cache = require( 'gulp-cache' );

var include = require( 'gulp-file-include' ); // включение файлов

var clean = require( 'gulp-clean' ); // удаление файлов и каталогов

var browserSync = require( 'browser-sync' ).create(); // обновление налету

// ================== CLEAN ============================
// Полная очистка
gulp.task( 'CLEAN', function( cb ){
	pump([
		gulp.src( [ './dev', './prod' ], { read: false } ),
		clean()
	], cb );
});


// ================== LESS ============================

// Собираем все CSS библиотеки
gulp.task( 'dev:css-libs', function( cb ){
	pump( [
		gulp.src( libCSS ),
		concat( 'vendor.css' ),
		gulp.dest( './dev/assets/css' )
	], cb );
});

// Собираем CSS
gulp.task( 'dev:css', [ 'dev:css-libs' ], function( cb ){
	// Настройки для вендорных префиксов
	var plugins = [
		postcssPresetEnv({
			stage: 0
		})
	];

	pump([
		gulp.src( './src/assets/less/*.less' ),
		sourcemaps.init(),
		postcss( plugins ),
		sourcemaps.write( '.' ),
		gulp.dest( './dev/assets/css' ),
		browserSync.stream()
	], cb );
});

// Собираем все CSS библиотеки
gulp.task( 'prod:css-libs', function( cb ){

	pump([
		gulp.src( libCSS ),
		concat( 'vendor.css' ),
		gulp.dest( './docs/assets/css' )
	], cb );
});

// Собираем CSS
gulp.task( 'prod:css', [ 'prod:css-libs' ], function( cb ){
	// Настройки для вендорных префиксов
	var plugins = [
		postcssPresetEnv({
			features: {
				'custom-properties': false
			},
			stage: 0,
			autoprefixer: { grid: true }
		}),
		cssvariables()
	];

	pump([
		gulp.src( './src/assets/css/*.css' ),
		postcss( plugins ),
		gulp.dest( './prod/assets/css' )
	], cb );
});

// ================== JS ============================
// Компилируем и собираем все js файлы проекта
gulp.task( 'dev:js', function(){

	// ES5
	pump([
		gulp.src( './src/assets/js/*.js' ),
		webpack( require( './webpack.dev.js' ), compiler, function( err, stats ){
			// console.log( err, stats );
		}),
		gulp.dest( './dev/assets/js' )
	]);

});


// Обрабатываем js файлы библиотек и проекта
gulp.task( 'prod:js', function(){

	// ES5
	pump([
		gulp.src( './src/assets/js/*.js' ),
		webpack( require( './webpack.prod.js' ), compiler, function( err, stats ){
			// console.log( err, stats );
		}),
		gulp.dest( './prod/assets/js' )
	]);

});


// ================== IMG ============================
// Оптимизация картинок для dev
gulp.task('dev:img', function( cb ) {
	pump([
		gulp.src( [
			'./src/assets/img/**/*',
			'./src/components/**/*.png',
			'./src/components/**/*.jpg',
			'./src/components/**/*.jpeg',
			'./src/components/**/*.gif',
			'./src/components/**/*.svg' ]
		),
		cache(
			imagemin()
		),
		gulp.dest( './dev/assets/img' )
	], cb );
});

// Оптимизация картинок для prod
gulp.task('prod:img', function( cb ) {
	pump([
		gulp.src( [
			'./src/assets/img/**/*',
			'./src/components/**/*.png',
			'./src/components/**/*.jpg',
			'./src/components/**/*.jpeg',
			'./src/components/**/*.gif',
			'./src/components/**/*.svg' ]
		),
		cache(
			imagemin()
		),
		gulp.dest( './docs/assets/img' )
	], cb );
});


// ================== HTML ============================
// HTML для dev
gulp.task( 'dev:html', function( cb ){

	// HTML
	pump([
		gulp.src( [
			'./src/**/*.html',
			'!./src/assets/*.html',
			'!./src/components/**/*.html']
		),
		include({
			prefix: '@@',
			basepath: '@file',
			context: {
				rev: +new Date
			}
		}),
		gulp.dest( './dev' )
	], cb );
});

// Статичные файлы
gulp.task( 'dev:static', function( cb ){
	pump([
		gulp.src( [
			'./src/favicon.ico',
			'./src/robots.txt',
			'./src/CNAME'
		] ),
		gulp.dest( './dev' )
	], cb );
});

// HTML для prod
gulp.task( 'prod:html', function( cb ){
	pump([
		gulp.src( ['./src/**/*.html', '!./src/assets/*.html', '!./src/components/**/*.html', './src/**/*.xml', './src/**/*.xsl'] ),
		include({
			prefix: '@@',
			basepath: '@file',
			context: {
				rev: +new Date
			}
		}),
		gulp.dest( './docs' )
	], cb );
});

// Статичные файлы
gulp.task( 'prod:static', function( cb ){
	pump([
		gulp.src( [
			'./src/favicon.ico',
			'./src/robots.txt',
			'./src/CNAME'
		] ),
		gulp.dest( './docs' )
	], cb );
});

// ================== DEV ============================
// Слежение за dev
gulp.task( 'DEV', [ 'dev:img', 'dev:css', 'dev:js', 'dev:html', 'dev:static' ], function() {

	browserSync.init({
		server: {
			baseDir: './dev/'
		}
	});

	gulp.watch( './src/assets/less/*.less' , ['dev:css']);
	gulp.watch( './src/components/**/*.less' , ['dev:css', 'dev:js']);

	gulp.watch( './src/assets/js/*.js', [ 'dev:js' ] );
	gulp.watch( './src/components/**/*.js', [ 'dev:js' ] );
	gulp.watch( './dev/assets/js/**/*' ).on('change', browserSync.reload);

	gulp.watch( './src/**/*.html', [ 'dev:html' ] );
	gulp.watch( './src/components/**/*.html', [ 'dev:html' ] );
	gulp.watch( './dev/*.html' ).on('change', browserSync.reload);

	gulp.watch( './src/assets/img/**/*', [ 'dev:img' ] );
	gulp.watch( './src/components/**/*.gif', [ 'dev:img' ] );
	gulp.watch( './src/components/**/*.jpg', [ 'dev:img' ] );
	gulp.watch( './src/components/**/*.jpeg', [ 'dev:img' ] );
	gulp.watch( './src/components/**/*.png', [ 'dev:img' ] );
	gulp.watch( './src/components/**/*.svg', [ 'dev:img' ] );
	gulp.watch( './dev/assets/img/**/*' ).on( 'change', browserSync.reload );
});

// ================== PROD ============================
// Публикация на prod
gulp.task( 'PROD', [ 'prod:clean' ], function(){
	gulp.start( 'prod:img', 'prod:css', 'prod:js', 'prod:html', 'prod:static' );
});